/**
 * @module dixim
 * @submodule autoDownload
 */
dixim.namespace('dixim.autoDownload');

/**
 * @module autoDownload
 * @submodule filter-edit-dialog-view
 */

(function() {
var FilterEditDialogView;
var prototype;
var superConstructor;
var _ = dixim.ctt.ResourceBundle.get;
var Event = dixim.event.Event;
var ModalDialogView = dixim.common.ModalDialogView;
var Enum = dixim.ctt.util.Enum;
var $ = dixim.ctt.jQuery;

/**
 * @class FilterEditDialogView
 * @constructor
 * @param {Object} configs
 */
FilterEditDialogView = function(configs) {
    ModalDialogView.call(this, configs);
};

dixim.inherit(FilterEditDialogView, ModalDialogView);
dixim.autoDownload.FilterEditDialogView = FilterEditDialogView;
prototype = FilterEditDialogView.prototype;
superConstructor = FilterEditDialogView.superConstructor;

var SETTING = 'setting';
var SELECT_CONDITION = 'selectCondition';

var TEMPLATE_BODY_CONTENT = [
    '<div class="none_margin_top dlg_scroll dl_tbl">',
    '<div id="auto-download-filter-info">',
    '<dl>',
    '<dt>', '${filterName}', '</dt>',
    '<dd>',
    '<input type="text" id="filter-add-filter-name" maxlength="255"/>',
    '</dd>',
    '</dl>',
    '<dl>',
    '<dt>', '${couplingConditions}', '</dt>',
    '<dd>',
    '<label class="select-pointer">',
    '<input type="radio" name="filter-couplings" id="filter-couplings-and" data-role="none" checked />',
    '<span>', '${couplingAnd}', '</span>',
    '</label>',
    '<label class="select-pointer">',
    '<input type="radio" name="filter-couplings" id="filter-couplings-or" data-role="none" />',
    '<span>', '${couplingOr}', '</span>',
    '</label>',
    '</dd>',
    '</dl>',
    '</div>',
    '<div id="auto-download-filter-coupling-conditions" class="coupling_conditions">',
    '<dl>',
    '<dt>', '${conditionTitle}', '</dt>',
    '<dd>', '</dd>',
    '</dl>',
    '</div>',
    '</div>',
    '<div id="auto-download-filter-error">', '</div>',
].join('');

var COND = Enum.fetch('COND');
var FilterMode = Enum.fetch('DIALOG');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    var buttonText;

    configs = configs || {};

    configs.id = configs.id || '';
    configs.id = [configs.id, 'filter-edit-dialog-'].join('');

    configs.bodyContent = TEMPLATE_BODY_CONTENT
        .replace('${filterName}', _('auto_download_filter_edit_filter_name'))
        .replace('${couplingConditions}', _('auto_download_filter_edit_coupling_conditions'))
        .replace('${couplingAnd}', _('auto_download_filter_edit_coupling_and'))
        .replace('${couplingOr}', _('auto_download_filter_edit_coupling_or'))
        .replace('${conditionTitle}', _('auto_download_filter_edit_condition_title_name'));

    configs.doubleDialog = true;
    configs.contentHeight = configs.contentHeight || '300px';
    switch(configs.mode) {
        case FilterMode.ADD:
            configs.title = _('auto_download_filter_add_title');
            buttonText = _('dialog_add');
            break;
        case FilterMode.EDIT:
            configs.title = _('auto_download_filter_edit_title');
            buttonText = _('dialog_edit');
            break;
    }
    configs.buttons = [
        {text: _('dialog_cancel'), click: dixim.proxy(this.hide, this)},
        {text: buttonText, click: dixim.proxy(this._clickSetting, this)},
    ];

    superConstructor.prototype._initialize.call(this, configs);
};

prototype._render = function() {
    var addCouplingConditions = dixim.proxy(function(condId, name) {
        $('#auto-download-filter-coupling-conditions dd').append([
            '<div id="cond_', condId, '">',
            '<button id="cond_button_', condId, '" class="condition-buttons select-pointer ui-btn ui-shadow ui-corner-all color_btn btn">', name, '</button>',
            '<div class="condition-text">',
            '<span>', '(', _('auto_download_filter_edit_setting_condition'), '</span>',
            '<span class="condition-status">', '</span>',
            '<span>', ')', '</span>',
            '</div>',
            '</div>',
        ].join(''));
    
        this.setConditionStatus(condId, false);
        $(['#cond_button_', condId].join('')).on('click', {condId: condId}, dixim.proxy(this._clickSelectCondition, this));
    }, this);

    $('#auto-download-filter-info input').on('blur', function() {
        setTimeout(function() {
            $('body').scrollTop($('body').scrollTop() + 1);
            $('body').scrollTop($('body').scrollTop() - 1);
        }, 100);
    });

    for (var key in COND) {
        addCouplingConditions(COND[key], _(['auto_download_filter_edit_condition_', COND[key]].join('')));
    }

    superConstructor.prototype._render.call(this);
};

prototype.setFilterInfo = function(filterInfo) {
    var conds = filterInfo.conds;

    $('#filter-add-filter-name').val(filterInfo.title);
    $('#filter-couplings-and').prop('checked', (filterInfo.combCond === 'AND'));
    $('#filter-couplings-or').prop('checked', (filterInfo.combCond === 'OR'));

    for (var key in conds) {
        this.setConditionStatus(key, (conds[key].length !== 0));
    }
};

prototype.setConditionStatus = function(condId, condition) {
    var conditionName = (condition) ? _('auto_download_filter_edit_setting_condition_yes') : _('auto_download_filter_edit_setting_condition_no');
    var className = (condition) ? 'condition-yes' : 'condition-no';
    var removeClassName = (condition) ? 'condition-no' : 'condition-yes';

    $(['#auto-download-filter-coupling-conditions div#cond_', condId, ' .condition-status'].join('')).removeClass(removeClassName);
    $(['#auto-download-filter-coupling-conditions div#cond_', condId, ' .condition-status'].join('')).addClass(className);
    $(['#auto-download-filter-coupling-conditions div#cond_', condId, ' .condition-status'].join('')).text(conditionName);
};

prototype._clickSelectCondition = function(e) {
    this.dispatchEvent(new Event(SELECT_CONDITION, {condId: e.data.condId}));
};

prototype._clickSetting = function() {
    var info = {};

    info.title = $('#filter-add-filter-name').val();
    info.combCond = $('#filter-couplings-and').prop('checked') ? 'AND' : 'OR';
    this.dispatchEvent(new Event(SETTING, info));
};

}());
