/**
 * @module dixim
 * @submodule autoDownload
 */
dixim.namespace('dixim.autoDownload');

/**
 * @module autoDownload
 * @submodule operation-menu-dialog-view
 */

(function() {
var OperationMenuDialogView;
var prototype;
var superConstructor;
var ModalDialogView = dixim.common.ModalDialogView;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var Enum = dixim.ctt.util.Enum;

/**
 * @class OperationMenuDialogView
 * @constructor
 * @param {Object} configs
 */
OperationMenuDialogView = function(configs) {
    ModalDialogView.call(this, configs);
};

dixim.inherit(OperationMenuDialogView, ModalDialogView);
dixim.autoDownload.OperationMenuDialogView = OperationMenuDialogView;
prototype = OperationMenuDialogView.prototype;
superConstructor = OperationMenuDialogView.superConstructor;

var ADD_TASK = 'addTask';
var CHANGE_TASK = 'changeTask';
var DELETE_TASK = 'deleteTask';
var CONFIRM_STATUS = 'confirmStatus';

var OperationMenu = Enum.fetch('OPERATION_MENU');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    var isOnFlag = function() {
        for (var i = 0, len = arguments.length; i < len; i += 1) {
            if (!Enum.isOnFlag(configs.operationStatus, OperationMenu[arguments[i]])) return false;
        }
        return true;
    };
    var isOffFlag = function() {
        for (var i = 0, len = arguments.length; i < len; i += 1) {
            if (!Enum.isOffFlag(configs.operationStatus, OperationMenu[arguments[i]])) return false;
        }
        return true;
    };

    configs.id = configs.id || '';
    configs.id = [configs.id, 'operation-menu-dialog-'].join('');

    configs.title = _('dialog_title_operation');
    configs.closeButton = dixim.proxy(this.hide, this);
    configs.buttons = [
        {text: _('dialog_close'), click: dixim.proxy(this.hide, this)}
    ];

    configs.lists = [];
    configs = this._addTaskAdd(configs);
    if (isOnFlag('TASK') && isOffFlag('MULTIPLE')) {
        configs = this._addTaskEdit(configs);
    }
    if (isOnFlag('TASK')) {
        configs = this._addTaskDel(configs);
    }
    if (isOnFlag('TASK') && isOffFlag('MULTIPLE')) {
        configs = this._addTaskState(configs);
    }

    superConstructor.prototype._initialize.call(this, configs);
};

prototype._addTaskAdd = function(configs) {
    configs.lists.push({text: _('auto_download_operation_add_task'), click: dixim.proxy(this._clickAddTask, this)});
    return configs;
};

prototype._addTaskEdit = function(configs) {
    configs.lists.push({text: _('auto_download_operation_change_task'), click: dixim.proxy(this._clickChangeTask, this)});
    return configs;
};

prototype._addTaskDel = function(configs) {
    configs.lists.push({text: _('auto_download_operation_delete_task'), click: dixim.proxy(this._clickDeleteTask, this)});
    return configs;
};

prototype._addTaskState = function(configs) {
    configs.lists.push({text: _('auto_download_operation_confirm_status'), click: dixim.proxy(this._clickConfirmStatus, this)});
    return configs;
};

prototype._clickAddTask = function() {
    this.dispatchEvent(new Event(ADD_TASK));
};

prototype._clickChangeTask = function() {
    this.dispatchEvent(new Event(CHANGE_TASK));
};

prototype._clickDeleteTask = function() {
    this.dispatchEvent(new Event(DELETE_TASK));
};

prototype._clickConfirmStatus = function() {
    this.dispatchEvent(new Event(CONFIRM_STATUS));
};

}());
