/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule mkdir-dialog-controller
 */
(function() {
var MkdirDialogController;
var prototype;
var superConstructor;
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var MkdirDialogView = dixim.contents.MkdirDialogView;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var _ = dixim.ctt.ResourceBundle.get;

/**
 * @class MkdirDialogController
 * @constructor
 * @param {Object} configs
 * @param {MobileTransferApiClient} [configs.model]
 */
MkdirDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

var MKDIR = 'mkdir';
var UPDATE = 'update';

dixim.inherit(MkdirDialogController, EventTarget);
dixim.contents.MkdirDialogController = MkdirDialogController;
prototype = MkdirDialogController.prototype;
superConstructor = MkdirDialogController.superConstructor;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.targetItem
 * @param {MobileTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._model = new ApiClientTransfer();
    this._model.addEventListener('makeDirectory', this._successMakeDirectory, this);
    this._model.addEventListener('makeDirectoryError', this._errorMakeDirectory, this);

    this._view = new MkdirDialogView(configs);
    this._view.addEventListener(MKDIR, this._clickMkdir, this);

    this._parentId = configs.parentId;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._successMakeDirectory = function() {
    this._view.stopIndicator();
    this.stop();
    this.dispatchEvent(new Event(UPDATE));
};

prototype._errorMakeDirectory = function(e) {
    var errorCode = e.args.errorCode;

    this._view.stopIndicator();
    switch(errorCode) {
        case 409:
            this._view.error(_(['contents_mkdir_error_', errorCode].join('')));
            break;
        default:
            this._view.error(_('contents_mkdir_error'));
            break;
    }
};

prototype._clickMkdir = function(e) {
    var query = {};

    this._view.startIndicator();
    query.name = e.args.name;
    query.parent_id = this._parentId;
    this._model.makeDirectory(query);
};

}());
