/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule operation-menu-dialog-controller
 */

(function() {
var OperationMenuDialogController;
var prototype;
var superConstructor;
var OperationMenuDialogView = dixim.contents.OperationMenuDialogView;
var EventTarget = dixim.event.EventTarget;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var RenameDialogController = dixim.contents.RenameDialogController;
var DeletionDialogController = dixim.contents.DeletionDialogController;
var ProgressDialogController = dixim.contents.ProgressDialogController;
var CompressDialogController = dixim.contents.CompressDialogController;
var TransferWizardDialogController = dixim.contents.TransferWizardDialogController;
var ConfirmModalDialogView = dixim.common.ConfirmModalDialogView;
var MoveDialogController = dixim.contents.MoveDialogController;
var MkdirDialogController = dixim.contents.MkdirDialogController;
var Locker = dixim.ctt.util.Locker;

var Enum = dixim.ctt.util.Enum;

/**
 * @class OperationMenuDialogController
 * @constructor
 * @param {Object} configs
 */
OperationMenuDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(OperationMenuDialogController, EventTarget);
dixim.contents.OperationMenuDialogController = OperationMenuDialogController;
prototype = OperationMenuDialogController.prototype;
superConstructor = OperationMenuDialogController.superConstructor;

var DELETE = 'delete';
var OK = 'ok';
var NOT_PERMITTED_AUTO_DELETE = 'notPermittedAutoDelete';
var PERMITTED_AUTO_DELETE = 'permittedAutoDelete';
var RENAME = 'rename';
var MOVE = 'move';
var MKDIR = 'mkdir';
var UPDATE = 'update';
var CLOSE = 'close';
var COMPRESS = 'compress';
var TRANSFER = 'transfer';

var Mode = Enum.fetch('PROGRESS');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'operation-menu-dialog'].join('');

    this._view = new OperationMenuDialogView(configs);
    this._view.addEventListener(DELETE, this._clickDelete, this);
    this._view.addEventListener(NOT_PERMITTED_AUTO_DELETE, this._clickNotPermittedAutoDelete, this);
    this._view.addEventListener(PERMITTED_AUTO_DELETE, this._clickPermittedAutoDelete, this);
    this._view.addEventListener(RENAME, this._clickRename, this);
    this._view.addEventListener(MOVE, this._clickMove, this);
    this._view.addEventListener(MKDIR, this._clickMkdir, this);
    this._view.addEventListener(COMPRESS, this._clickCompress, this);
    this._view.addEventListener(TRANSFER, this._clickTransfer, this);

    this._dataList = configs.dataList || [];
    this._parentId = configs.parentId;
    this._browsingStackObj = configs.browsingStackObj;
    this._locker = new Locker();
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._isIncludeContainer = function(dataList) {
    var dataListLen = dataList.length;

    for (var i = 0; i < dataListLen; i += 1) {
        if (dataList[i].class === 'container') {
            return true;
        }
    }
    return false;
};

prototype._getExceptContainerData = function(dataList) {
    var tmpDataList = $.extend([], dataList);

    for (var i = 0; i < tmpDataList.length; i += 1) {
        if (tmpDataList[i].class === 'container') {
             tmpDataList.splice(i, 1);
             i -= 1;
        }
    }
    return tmpDataList;
};

prototype._clickDelete = function() {
    var controller;
    var query = {};

    if (this._dataList.length === 0) {
        this._errorDialog(_('contents_operation_menu_delete'), _('contents_operation_menu_error_no_selected'))
        return;
    }
    query.contents = this._dataList;
    query.doubleDialog = true;
    controller = new DeletionDialogController(query);
    controller.addEventListener(UPDATE, this._callbackUpdate, this);
    controller.start();
};

prototype._clickNotPermittedAutoDelete = function() {
    var callback = dixim.proxy(function() {
        var query = {};
        var controller;
        var dataList = this._getExceptContainerData(this._dataList);

        if (dataList.length === 0) {
            this._errorDialog(_('contents_operation_menu_delete'), _('contents_operation_menu_error_no_selected'))
            return;
        }
        query.contents = dataList;
        query.mode = Mode.NOT_PERMITTED;
        query.doubleDialog = true;
        controller = new ProgressDialogController(query);
        controller.addEventListener(UPDATE, this._callbackUpdate, this);
        controller.start();
    }, this);

    if (this._isIncludeContainer(this._dataList)) {
        this._confirmDialog(_('contents_operation_menu_delete'), _('contents_confirm_description'), callback);
    } else {
        callback();
    }
};

prototype._clickPermittedAutoDelete = function() {
    var callback = dixim.proxy(function() {
        var controller;
        var query = {};
        var dataList = this._getExceptContainerData(this._dataList);

        if (dataList.length === 0) {
            this._errorDialog(_('contents_operation_menu_delete'), _('contents_operation_menu_error_no_selected'))
            return;
        }
        query.contents = dataList;
        query.mode = Mode.PERMITTED;
        query.doubleDialog = true;
        controller = new ProgressDialogController(query);
        controller.addEventListener(UPDATE, this._callbackUpdate, this);
        controller.start();
    }, this);

    if (this._isIncludeContainer(this._dataList)) {
        this._confirmDialog(_('contents_operation_menu_delete'), _('contents_confirm_description'), callback);
    } else {
        callback();
    }
};

prototype._clickRename = function() {
    var controller;
    var query = {};

    if (this._dataList.length !== 1) {
        this._errorDialog(_('contents_operation_menu_rename'), _('contents_operation_menu_error_not_one'))
        return;
    }
    query.targetItem = this._dataList[0];
    query.doubleDialog = true;
    controller = new RenameDialogController(query);
    controller.addEventListener(UPDATE, this._callbackUpdate, this);
    controller.start();
};

prototype._clickMove = function() {
    var controller;
    var query = {};

    if (this._dataList.length === 0) {
            this._errorDialog(_('contents_operation_menu_move'), _('contents_operation_menu_error_no_selected'))
        return;
    }
    query.contents = this._dataList;
    query.doubleDialog = true;
    query.browsingStackObj = this._browsingStackObj;
    controller = new MoveDialogController(query);
    controller.addEventListener(UPDATE, this._callbackUpdate, this);
    controller.start();
};

prototype._clickMkdir = function() {
    var controller;
    var query = {};

    query.doubleDialog = true;
    query.parentId = this._parentId;
    controller = new MkdirDialogController(query);
    controller.addEventListener(UPDATE, this._callbackUpdate, this);
    controller.start();
};

prototype._clickCompress = function() {
    var callback = dixim.proxy(function() {
        var controller;
        var query = {};
        var dataList = this._getExceptContainerData(this._dataList);

        if (dataList.length === 0) {
            this._errorDialog(_('contents_operation_menu_compress'), _('contents_operation_menu_error_no_selected'))
            return;
        }
        query.contents = dataList;
        controller = new CompressDialogController(query);
        controller.addEventListener(UPDATE, this._callbackUpdate, this);
        controller.start();
    }, this);

    if (this._isIncludeContainer(this._dataList)) {
        this._confirmDialog(_('contents_operation_menu_compress'), _('contents_confirm_description'), callback);
    } else {
        callback();
    }
};

prototype._clickTransfer = function() {
    var callback = dixim.proxy(function() {
        var controller;
        var query = {};
        var dataList = this._getExceptContainerData(this._dataList);

        if (dataList.length === 0) {
            this._errorDialog(_('contents_operation_menu_transfer'), _('contents_operation_menu_error_no_selected'))
            return;
        }
        query.dataList = dataList;
        controller = new TransferWizardDialogController(query);
        controller.addEventListener(UPDATE, this._callbackUpdate, this);
        controller.addEventListener(UPDATE, this._closeDialog, this);
        controller.addEventListener(CLOSE, this._closeDialog, this);
        controller.start();
    }, this);

    if (this._locker.isLock()) return;
    this._locker.lock();

    if (this._isIncludeContainer(this._dataList)) {
        this._confirmDialog(_('contents_operation_menu_transfer'), _('contents_confirm_description'), callback);
    } else {
        callback();
    }
};

prototype._closeDialog = function() {
    this._locker.unlock();
};

prototype._callbackUpdate = function() {
    this.stop();
    this.dispatchEvent(new Event(UPDATE));
};

prototype._confirmDialog = function(title, text, callback) {
    var query = {};
    var dialog;
    var clickOk = dixim.proxy(function() {
        dialog.hide();
        callback();
    }, this);

    query.id = this._id;
    query.title = title;
    query.description = text;
    query.doubleDialog = true;
    dialog = new ConfirmModalDialogView(query);
    dialog.addEventListener(OK, clickOk, this);
    dialog.show();
};

prototype._errorDialog = function(title, text) {
    var query = {};
    var dialog;

    query.id = this._id; 
    query.title = title;
    query.description = text;
    query.doubleDialog = true;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
