/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule transfer-reservation-dialog-view
 */

(function() {
var TransferReservationDialogView;
var prototype;
var superConstructor;
var Event = dixim.event.Event;
var ModalDialogView = dixim.common.ModalDialogView;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;
var Formatter = dixim.ctt.util.Formatter;

/**
 * @class TransferReservationDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
TransferReservationDialogView = function(configs) {
    ModalDialogView.call(this, configs);
};

dixim.inherit(TransferReservationDialogView, ModalDialogView);
dixim.contents.TransferReservationDialogView = TransferReservationDialogView;
prototype = TransferReservationDialogView.prototype;
superConstructor = TransferReservationDialogView.superConstructor;

var TEMPLATE_BODY_CONTENT = [
    '<div id="transfer-reservation-view" class="none_margin_top dlg_scroll">',
    '<table id="ctt-transfer-reservation-list">',
    '<colgroup>',
    '<col span="1" class="checkbox">', '<col span="1" class="list_info">',
    '</colgroup>',
    '</table>',
    '</div>',
].join('');

var CHANGE = 'change';
var DELETE = 'delete';

prototype._initialize = function(configs) {
    configs  = configs || {};

    configs.id = configs.id || '';
    configs.id = [configs.id, 'transfer-reservation-dialog-'].join('');

    configs.buttons = [
        {
            text: _('dialog_return_menu'),
            click: dixim.proxy(this.hide, this)
        },
        {
            text: _('dialog_edit'),
            click: dixim.proxy(this._clickChange, this)
        },
        {
            text: _('dialog_delete'),
            click: dixim.proxy(this._clickDelete, this)
        },
    ];

    configs.title = _('contents_setting_menu_check_reservation');
    configs.bodyContent = TEMPLATE_BODY_CONTENT;
    configs.contentHeight = '300px';
    configs.doubleDialog = true;

    this._type = configs.type;

    superConstructor.prototype._initialize.call(this, configs);
};

prototype.addReservationList = function(opts) {
    var date = Formatter.stringifyDate(Formatter.dcDateToDate(opts.startTime), 'YYYY/MM/DD hh/mm/ss');

    $('#ctt-transfer-reservation-list').append([
        '<tbody id="reservation-', opts.id, '" class="reservation-list">',
        '<tr>', '<td>',
        '<label>',
        '<input type="checkbox" id="', opts.id, '">',
        '<span class="reservation-btn">', '</span>',
        '</label>',
        '</td>',
        '<td class="transfer-reservation-title-field">',
        '<span id="reservation-title" class="list_title_s">', '</span>',
        '<div class="list_detail txt_s">',
        '<dl class="list_4em">',
        '<dt>', (this._type === 'upload') ? _('contents_transfer_reservation_dialog_server') : _('contents_transfer_reservation_dialog_src_server'), ' : ', '</dt>',
        '<dd>', '<p>', '<span id="reservation-server" class="reservation-description">', '</span>', '</p>', '</dd>',
        '<dt>', _('contents_transfer_reservation_dialog_date'), ' : ', '</dt>',
        '<dd>', '<p>', '<span id="reservation-date" class="reservation-description">', date, '</span>', '</p>', '</dd>',
        '</dl>',
        '</div>',
        '</td>', '</tr>',
        '</tbody>',
    ].join(''));
    $(['#reservation-', opts.id, ' #reservation-title'].join('')).text(opts.title);
    $(['#reservation-', opts.id, ' #reservation-server'].join('')).text(opts.server);
};

prototype.addNoContents = function() {
    this.error(_('contents_transfer_reservation_dialog_no_content'));
};

prototype.setEventButtons = function() {
    $('#ctt-transfer-reservation-list [type="checkbox"]').on('change', dixim.proxy(this.checkReservationOperate, this));
};

prototype.checkReservationOperate = function() {
    var checked = $('#ctt-transfer-reservation-list :checked').length;

    if (checked === 1) {
        this.enableButton(_('dialog_edit'));
    } else {
        this.disableButton(_('dialog_edit'));
    }

    if (checked > 0) {
        this.enableButton(_('dialog_delete'));
    } else {
        this.disableButton(_('dialog_delete'));
    }
};

prototype.clearTransferHistoryList = function() {
    $('#ctt-transfer-reservation-list').empty();
};

prototype._clickChange = function() {
    var id = $('#ctt-transfer-reservation-list :checked').prop('id');

    this.dispatchEvent(new Event(CHANGE, id));
};

prototype._clickDelete = function() {
    var dataList = [];

    $('#ctt-transfer-reservation-list :checked').each(function() {
        var data;

        data = {
            id: $(this).prop('id'),
            title: $(this).closest('tbody').find('#reservation-title').text(),
            server: $(this).closest('tbody').find('#reservation-server').text(),
            date: $(this).closest('tbody').find('#reservation-date').text(),
        };
        dataList.push(data);
    });
    this.dispatchEvent(new Event(DELETE, dataList));
};

prototype.error = function(text) {
    $('#transfer-reservation-no-content').remove();

    $('#transfer-reservation-view').append([
        '<div id="transfer-reservation-no-content" class="txt_none error-text-field">', '<span>', text, '</span>', '</div>',
    ].join(''));
};

}());
