/**
 * @module dixim
 * @submodule download
 */
dixim.namespace('dixim.download');

/**
 * @module download
 * @submodule setting-menu-dialog-controller
 */

(function() {
var SettingMenuDialogController;
var prototype;
var superConstructor;
var SettingMenuDialogView = dixim.download.SettingMenuDialogView;
var EventTarget = dixim.event.EventTarget;
var TransferHistoryDialogController = dixim.contents.TransferHistoryDialogController;
var TransferReservationDialogController = dixim.contents.TransferReservationDialogController;

/**
 * @class SettingMenuDialogController
 * @constructor
 * @param {Object} configs
 */
SettingMenuDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(SettingMenuDialogController, EventTarget);
dixim.download.SettingMenuDialogController = SettingMenuDialogController;
prototype = SettingMenuDialogController.prototype;
superConstructor = SettingMenuDialogController.superConstructor;

var HISTORY = 'history';
var CHECK_RESERVATION = 'checkReservation';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'setting-menu-dialog'].join('');

    this._view = new SettingMenuDialogView(configs);
    this._view.addEventListener(HISTORY, this._clickHistory, this);
    this._view.addEventListener(CHECK_RESERVATION, this._clickCheckReservation, this);
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._clickHistory = function() {
    var query = {};
    var controller;

    query.direction = 'download';
    controller = new TransferHistoryDialogController(query);
    controller.start();
};

prototype._clickCheckReservation = function() {
    var query = {};
    var controller;

    query.type = 'download';
    controller = new TransferReservationDialogController(query);
    controller.start();
};

}());
