/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule select-src-container-dialog-controller
 */

(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var SelectSrcContainerDialogView = dixim.ctt.ad.SelectSrcContainerDialogView;
var AutoDownloadApiClient = dixim.ctt.ad.AutoDownloadApiClient;

var SelectSrcContainerDialogController;
var prototype;

/**
 * @class SelectSrcContainerDialogController
 * @constructor
 * @param {Object} configs
 * @param {AutoDownloadApiClient} [configs.model]
 */
SelectSrcContainerDialogController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(SelectSrcContainerDialogController, EventTarget);
dixim.ctt.ad.SelectSrcContainerDialogController = SelectSrcContainerDialogController;
prototype = SelectSrcContainerDialogController.prototype;

/**
 * Triggered when the deletion has been done.
 *
 * @event close
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.deleted true if the deletion is completed, otherwise false.
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._model = new AutoDownloadApiClient();
  this._view = new SelectSrcContainerDialogView(configs);
  this._session = null;
  this._queue = [];
};

/**
 * @method start
 */
prototype.start = function(query) {
  var v = this._view;
  this._finish = query.finish || null;

  v.addEventListener('next', this._didClickNext, this);
  v.addEventListener('cancel', this._didClickCancel, this);

  v.show();
};

prototype._didClickNext = function() {
  this.destroy();
  this._finish(this._view.getSelectInfo());
};

prototype._didClickCancel = function() {
  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE));
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.destroy();
};

}());
