/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule select-howto-download-dialog-edit-controller
 */

(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var SelectHowtoDownloadDialogView = dixim.ctt.ad.SelectHowtoDownloadDialogView;
var FilterSettingDialogEditController = dixim.ctt.ad.FilterSettingDialogEditController;
var AutoDownloadApiClient = dixim.ctt.ad.AutoDownloadApiClient;
var ErrorDialog = dixim.ctt.ErrorDialog;
var _ = dixim.ctt.ResourceBundle.get;

var SelectHowtoDownloadDialogEditController;
var prototype;

/**
 * @class SelectHowtoDownloadDialogEditController
 * @constructor
 * @param {Object} configs
 * @param {AutoDownloadApiClient} [configs.model]
 */
SelectHowtoDownloadDialogEditController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(SelectHowtoDownloadDialogEditController, EventTarget);
dixim.ctt.ad.SelectHowtoDownloadDialogEditController = SelectHowtoDownloadDialogEditController;
prototype = SelectHowtoDownloadDialogEditController.prototype;

/**
 * Triggered when the deletion has been done.
 *
 * @event close
 * @param {Event} event
 * @param {Object} event.args
 * @param {Object} event.args.deleted true if the deletion is completed, otherwise false.
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._model = new AutoDownloadApiClient();
  this._view = new SelectHowtoDownloadDialogView(configs);
  this._session = null;
  this._queue = [];
  this._finish = configs.finish || function() {return;};
  this._select = configs.select || {};
};

/**
 * @method start
 */
prototype.start = function() {
  var v = this._view;

  v.addEventListener('submit', this._didClickSubmit, this);
  v.addEventListener('cancel', this._didClickCancel, this);
  v.addEventListener('edit', this._didClickEdit, this);
  v.addEventListener('delete', this._didClickDelete, this);
  v.addEventListener('allselect', this._didClickAllSelect, this);
  v.addEventListener('allunselect', this._didClickAllUnselect, this);

  v.show();
  this._didCreateDialog();
};

prototype._didCreateDialog = function() {
  var minute;
  var hour;
  var filter_list_query = {};
  var selected = this._select;
  var addTimedSelect = function() {
    var hour_option = "";
    var minute_option = "";

    for (var i = 0; i < 24; i += 1) {
      hour_option += ['<option value=',(("00"+i).slice(-2)),'>',i,'</option>'].join('');
    }
    for (var j = 0; j < 60; j += 10) {
      minute_option += ['<option value=',(("00"+j).slice(-2)),'>',(("00"+j).slice(-2)),'</option>'].join('');
    }

    $('#timed_selected').html(
        [
         '<select name="select_hour">',
         hour_option,
         '</select>',
         '<span id="hour_str">',_("ctt_ad_wizard_hour_str"),'</span>',
         '<select name="select_minute">',
         minute_option,
         '</select>',
         '<span id="minute_str">',_("ctt_ad_wizard_minute_str"),'</span>'
         ].join('')
    );
  };

  $('.ui-dialog').hide();
  $('.ctt-how-to-back').remove();
  $('.ui-dialog-buttonset').append(this._loadingIcon);
  $('#sd_resolution_checkbox').prop('checked', this._select.sdFirst);
  if (this._select.reservationType === "timed") {
    $("#timed_download_radio").prop('checked', true);
    $('#timed_selected').show();
    addTimedSelect();
    minute = ("00" + this._select.timedReservationStartTime % 60).slice(-2);
    hour = ("00" + Math.floor(this._select.timedReservationStartTime / 60)).slice(-2);
    $('select[name="select_hour"]').val(hour);
    $('select[name="select_minute"]').val(minute);
  }

  filter_list_query.success = dixim.proxy(function(e) {
    var filters = e.args.filters;
    var filters_length = filters.length;
    var taskIds_length;

    this._select.filterList = [];
    for (var i = 0; i < filters_length; i += 1) {
      taskIds_length = filters[i].taskIds.length;
      for (var j = 0; j < taskIds_length; j += 1) {
        if (selected.taskId === filters[i].taskIds[j]) {
          this._select.filterList.push(filters[i].filterId);
          $('#select_filter_list_table [type="checkbox"]:last').prop('checked', true).next().prop('disabled', false).val(filters[i].filterId);
          this._view._didAddFilterListRow();
        }
      }
    }
  }, this);
  filter_list_query.error = dixim.proxy(function() {
    this._showErrorDialog("ctt_error_dialog_default");
  }, this);
  filter_list_query.complete = dixim.proxy(function() {
    $('.ui-dialog').show();
  }, this);

  this._model.fetchDownloadFilterList(filter_list_query);
};

prototype._didClickSubmit = function() {
  var select_info  = this._view.getSelectInfo();
  var fetch_filter_list = this._select.filterList;
  var fetch_filter;
  var after_filter_list = select_info.filterList;
  var after_filter;
  var task_mod_query = {taskId: this._select.taskId};
  var filter_link_query = {taskId: this._select.taskId};
  var time;
  var filter_link_error = dixim.proxy(function() {
      this._showErrorDialog("ctt_error_dialog_default");
    }, this);

  task_mod_query.sdFirst = select_info.sdFirst;
  task_mod_query.reservationType = select_info.reservationType;
  if (select_info.reservationType === "TIMED") {
    time = parseInt($('[name="select_hour"]').val(), 10) * 60 + parseInt($('[name="select_minute"]').val(), 10);
    task_mod_query.timedReservationStartTime = time;
  }
  task_mod_query.error = dixim.proxy(function() {
    this._showErrorDialog("ctt_error_dialog_default");
  }, this);

  this._model.updateTask(task_mod_query);

  for (var i = 0; i < fetch_filter_list.length; i += 1) {
    fetch_filter = fetch_filter_list[i];
    for (var j = 0; j < after_filter_list.length; j += 1) {
      after_filter = after_filter_list[j];
      if (fetch_filter === after_filter) {
        fetch_filter_list.splice(i,1);
        i -= 1;
        after_filter_list.splice(j,1);
        j -= 1;
       }
     }
  }

  for (var k = 0; k < fetch_filter_list.length; k += 1) {
    filter_link_query.filterId = fetch_filter_list[k];
    filter_link_query.error = filter_link_error;
    this._model.unlinkFilter(filter_link_query);
  }
  for (var l = 0; l < after_filter_list.length; l += 1) {
    filter_link_query.filterId = after_filter_list[l];
    filter_link_query.error = filter_link_error;
    this._model.linkFilter(filter_link_query);
  }
  this.destroy();
  this._finish();
};

prototype._didClickCancel = function() {
  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE));
};

prototype._didClickEdit = function() {
  var query = {};
  var edit_dialog;
  var checkAndChangeName = function(name) {
    var new_name = '';
    var name_length = 0;
    var name_array = name.split('');
    var check_length = function(c) {
      if (c.match(/[a-z0-9]/)) {
        return 1;
      } else {
        return 1.8;
      }
    };

    for (var i = 0; i < name_array.length; i += 1) {
      name_length += check_length(name_array[i]);
      if (name_length < 55) {
        new_name += name_array[i];
      } else {
        new_name += "...";
        break;
      }
    }
    return new_name;
  };

  if ($('#select_filter_list_table [type="checkbox"]:checked').length === 1) {
    query.filterId = $('#select_filter_list_table [type="checkbox"]:checked').next().val();
    query.afterClose = dixim.proxy(function(data) {
      if (typeof data.filterName !== "undefined") {
        $(['#select_filter_list_table option[value="',data.filterId,'"]'].join('')).remove();
        $('#select_filter_list_table option[value="new"]').before(['<option value="',data.filterId,'">',checkAndChangeName(data.filterName),'</option>'].join(''));
        $('#select_filter_list_table [type="checkbox"]:checked').each(function() {
           $(this).next().val(data.filterId);
         });
       }
    }, this);
    edit_dialog = new FilterSettingDialogEditController(query);
    edit_dialog.start();
  } else {
    this._showErrorDialog('error_dialog_does_not_change_one_filter');
  }
};

prototype._didClickDelete = function() {
  $('#select_filter_list_table [type="checkbox"]:checked').closest('tr').remove();
};

prototype._didClickAllSelect = function() {
  $('#select_filter_list_table [type="checkbox"]').not(":last").prop("checked", true).trigger('change');
};

prototype._didClickAllUnselect = function() {
  $('#select_filter_list_table [type="checkbox"]').prop("checked", false).trigger('change');
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.destroy();
};

/**
 * @method _showErrorDialog
 * @private
 * @param {String} error error ID.
 */
prototype._showErrorDialog = function(error) {
  var dialog;
  var query = {type: error};

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  if (typeof error.title !== "undefined") {
    query.title = error.title;
  }
  if (typeof error.description !== "undefined") {
    query.description = error.description;
  }

  dialog = new ErrorDialog(query);
  dialog.addEventListener('close', this._didCloseModalDialog, this);

  dialog.show();
  this._modal = dialog;
};

prototype._didCloseModalDialog = function(e) {
  var args = e.args;

  if (this._modal === args.dialog) {
    this._modal.destroy();
    this._modal = null;
  }
};

}());
