/**
 * @module ctt
 * @submodule autodelete-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var AutoDeleteView = dixim.ctt.AutoDeleteView;
var AutoDeleteApiClient = dixim.ctt.api.AutoDeleteApiClient;
var _ = dixim.ctt.ResourceBundle.get;

var AutoDeleteController;
var prototype;

/**
 * @class AutoDeleteController
 * @constructor
 * @param {Object} configs
 * @param {Object} configs.targetItems
 * @param {ContentsTransferApiClient} [configs.model]
 */
AutoDeleteController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(AutoDeleteController, EventTarget);
dixim.ctt.AutoDeleteController = AutoDeleteController;
prototype = AutoDeleteController.prototype;

/**
 * Triggered when the autodelete has been done.
 *
 * @event close
 * @param {Event} event
 * @param {Object} event.args
 */
var CLOSE = 'close';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {ContentsTransferApiClient} [configs.model]
 */
prototype._initialize = function(configs) {
  configs = configs || {};

  this._model = configs.model || new AutoDeleteApiClient();
  this._view = new AutoDeleteView({});
  this._session = this._model.getstatus();
};

/**
 * @method start
 */
prototype.start = function() {
  var v = this._view;
  var m = this._model;

  v.addEventListener('autodelete', this._didClickAutoDelSetting, this);
  v.addEventListener('cancel', this._didClickCancel, this);

  m.addEventListener('setAutoDeleteStatus', this._didSetAutoDeleteStatus, this);
  m.addEventListener('setAutoDeleteStatusError', this._didSetAutoDeleteStatusError, this);

  m.addEventListener('getAutoDeleteStatus', this._didGetAutoDeleteStatus, this);
  m.addEventListener('getAutoDeleteStatusError', this._didGetAutoDeleteStatusError, this);

  m.addEventListener('setAutoDeleteLimit', this._didSetAutoDeleteLimit, this);
  m.addEventListener('setAutoDeleteLimitError', this._didSetAutoDeleteLimitError, this);

  m.addEventListener('getAutoDeleteLimit', this._didGetAutoDeleteLimit, this);
  m.addEventListener('getAutoDeleteLimitError', this._didGetAutoDeleteLimitError, this);

  v.show();
};

prototype._didClickAutoDelSetting = function(e) {
  var v = this._view;
  var autoDeleteLimit = v.getAutoDeleteLimit();

  this._session = this._model.setlimit({freeSpaceThreshold: autoDeleteLimit});
  v.startIndicator();
};

prototype._didClickCancel = function(e) {
  if (!!!this._session) {
    this._view.destroy();
    this.dispatchEvent(new Event(CLOSE));
    return;
  }
};

/* STATUS */
prototype._didSetAutoDeleteStatus = function(e) {
  this._session = null;
  this._view.destroy();
  this.dispatchEvent(new Event(CLOSE));
};

prototype._didSetAutoDeleteStatusError = function(e) {
  this._session = null;
  this._view.showError(_('ctt_autodelete_view_error'));
};

prototype._didGetAutoDeleteStatus = function(e) {
  var args = e.args;
  var data = args.data;
  var v = this._view;

  v.setAutoDeleteStatus(data.availability);
  this._session = this._model.getlimit();
};

prototype._didGetAutoDeleteStatusError = function(e) {
  this._session = null;
  this._view.showError(_('ctt_autodelete_view_error'));
};

/* LIMIT */
prototype._didSetAutoDeleteLimit = function(e) {
  this._session = null;
  var v = this._view;
  var autoDeleteStatus = v.getAutoDeleteStatus();
  this._session = this._model.setstatus({availability: autoDeleteStatus ? true : false});
};

prototype._didSetAutoDeleteLimitError = function(e) {
  this._session = null;
  this._view.showError(_('ctt_autodelete_view_error'));
};

prototype._didGetAutoDeleteLimit = function(e) {
  var args = e.args;
  var data = args.data;
  var v = this._view;

  this._session = null;
  v.setAutoDeleteLimit(data.freeSpaceThreshold);
};

prototype._didGetAutoDeleteLimitError = function(e) {
  this._session = null;
  this._view.showError(_('ctt_autodelete_view_error'));
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._view.hide();
};

/**
 * @method destroy
 */
prototype.destroy = function() {
  this._view.destroy();
};

}());
