
/**
 * @module ctt
 * @submodule download-move-in-wizard-controller
 */
(function() {
var WizardPageStackController = dixim.ctt.WizardPageStackController;
var DownloadDestinationWizardPageController = dixim.ctt.DownloadDestinationWizardPageController;

var DownloadMoveInWizardController;
var prototype;
var superConstructor;

/**
 * Provides a wizard to collect information for
 * transferring the contents specified.
 *
 * @class DownloadMoveInWizardController
 * @extends WizardPageStackController
 * @constructor
 * @param {Object} configs
 * @param {Object[]} configs.targetItems
 * @param {ContentsTransferApiClient} configs.model
 * @param {Boolean} [configs.enableWol=true] enables/disables WOL related functions which handle off-line DMSs.
 * @param {Boolean} [configs.enableScheduledTransferring=true] enables/disables Scheduled Transferring related functions.
 */
DownloadMoveInWizardController = function(configs) {
  WizardPageStackController.call(this, configs);
};
dixim.inherit(DownloadMoveInWizardController, WizardPageStackController);
dixim.ctt.DownloadMoveInWizardController = DownloadMoveInWizardController;
prototype = DownloadMoveInWizardController.prototype;
superConstructor = DownloadMoveInWizardController.superConstructor;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {Object} configs.dms
 * @param {Object[]} configs.targetItems
 * @param {ContentsTransferApiClient} configs.model
 * @param {Boolean} [configs.enableWol=true] enables/disables WOL related functions which handle off-line DMSs.
 * @param {Boolean} [configs.enableScheduledTransferring=true] enables/disables Scheduled Transferring related functions.
 */
prototype._initialize = function(configs) {
  var cfgs = {
    enableWol: true,
    enableScheduledTransferring: true
  };

  cfgs = dixim.extend(cfgs, configs);

  superConstructor.prototype._initialize.call(this, cfgs);

  this.setRegistryEntry('isExistMoveItem', false);
  this.setRegistryEntry('isUnknownWriteStatus', false);
  this.setRegistryEntry('isDownloadMoveIn', true);
  this.setRegistryEntry('sourceDevice', cfgs.sourceDevice);
  this.setRegistryEntry('targetItems', cfgs.targetItems);
  this.setRegistryEntry('model', cfgs.model);
  this.setRegistryEntry('enableWol', cfgs.enableWol);
  this.setRegistryEntry('enableScheduledTransferring', cfgs.enableScheduledTransferring);
  this._rootController = new DownloadDestinationWizardPageController();
  for (var i = 0; i < cfgs.targetItems.length; i += 1) {
    if (typeof cfgs.targetItems[i].copyCount !== "undefined") {
      this.setRegistryEntry('isExistMoveItem', true);
    }
    if (cfgs.targetItems[i].writeStatus && cfgs.targetItems[i].writeStatus === "UNKNOWN") {
      this.setRegistryEntry('isUnknownWriteStatus', true);
    }
  }
};

}());
