/**
 * @module ctt
 * @submodule ctt-ad
 */
dixim.namespace('dixim.ctt.ad');

/**
 * @module ctt-ad
 * @submodule select-dst-container-dialog-view
 */

(function() {
var Event = dixim.event.Event;
var ModalDialogView = dixim.ctt.ModalDialogView;
var AutoDownloadApiClient = dixim.ctt.ad.AutoDownloadApiClient;
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;
var Template = dixim.ctt.util.Template;
var _ = dixim.ctt.ResourceBundle.get;
var $ = dixim.ctt.jQuery;
var ErrorDialog = dixim.ctt.ErrorDialog;

var SelectDstContainerDialogView;
var prototype;
var superConstructor;

/**
 * @class SelectDstContainerDialogView
 * @extends ModalDialogView
 * @constructor
 * @param {Object} configs
 */
SelectDstContainerDialogView = function(configs) {
  ModalDialogView.call(this, configs);
};
dixim.inherit(SelectDstContainerDialogView, ModalDialogView);
dixim.ctt.ad.SelectDstContainerDialogView = SelectDstContainerDialogView;
prototype = SelectDstContainerDialogView.prototype;
superConstructor = SelectDstContainerDialogView.superConstructor;

var TEMPLATE_CONTENT_BOX = '<div id="select-dst-container"/>';
var TEMPLATE_LOADING = Template.LOADING_ICON_SMALL;

var TEMPLATE_BODY_CONTENT = ['<h1 id="task-add-h1">',
                             '</h1>',
                             '<div id="auto-download-tool-controller-server">',
                             '</div>'
                             ].join('');

/**
 * Triggered when the cancel button was clicked.
 *
 * @event cancel
 */
var CANCEL = 'cancel';

/**
 * Triggered when the next button was clicked.
 *
 * @event next
 */
var NEXT = 'next';

/**
 * Triggered when the back button was clicked.
 *
 * @event back
 */
var BACK = 'back';


/**
 * Triggered when the browse action was succeeded.
 *
 * @event browse
 */
var BROWSE = 'browse';

/**
 * Triggered when the browse action was failed.
 *
 * @event browseError
 */
var BROWSE_ERROR = 'browseError';

/**
 * Triggered when the storage_list action was succeeded.
 *
 * @event storageList
 */
var STORAGE_LIST = 'storageList';

/**
 * Triggered when the storage_list action was failed.
 *
 * @event storageListError
 */
var STORAGE_LIST_ERROR = 'storageListError';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  var buttons;
  var buttonsError;
  var bodyContent;

  configs  = configs || {};

  buttons = [
    {
      'text': _('ctt_transfer_wizard_button_label_next'),
      id: 'next',
      name: 'next',
      click: $.proxy(this._didClickNext, this),
      align: 'right',
      disabled: true
    },
    {
      'text': _('ctt_transfer_wizard_button_label_back'),
      id: 'back',
      click: $.proxy(this._didClickBack, this),
      align: 'right'
    },
    {
      'text': _('ctt_dialog_button_label_cancel'),
      id: 'cancel',
      click: $.proxy(this._didClickCancel, this),
      align: 'left'
    }
  ];

  buttonsError = [
    {
      'text': _('ctt_dialog_button_label_close'),
      click: $.proxy(this._didClickClose, this)
    }
  ];

  configs.title = configs.title || _('ctt_ad_wizard_select_dst_container');
  configs.contentBox = TEMPLATE_CONTENT_BOX;
  configs.bodyContent = TEMPLATE_BODY_CONTENT;
  configs.buttons = buttons;
  configs.width = 600;
  configs.height = 580;

  this._model = new AutoDownloadApiClient();
  this._model_transfer = new ContentsTransferApiClient();
  this._loadingIcon = $(TEMPLATE_LOADING);
  this._errorNode = null;
  this._errorButtons = buttonsError;
  this.result = {};
  this._rootContainerNames = [];
  this.checkObjectId = new this._didObjectId();
  this._remain_browse_storage_count = 0;

  this._model_transfer.addEventListener(BROWSE, dixim.proxy(this._didCreateDstRootContainerTableBrowse, this), this);
  this._model_transfer.addEventListener(STORAGE_LIST, dixim.proxy(this._didCreateDstRootContainerTableBrowseStorages, this), this);

  superConstructor.prototype._initialize.call(this, configs);
};

prototype._didSelectOneCheckbox = function() {
  var length = $('#auto-download-tool-controller-server-list input:checked').length;

  if (length !== 1) {
    this.disableButton($('#next'));
  } else {
    this.enableButton($('#next'));
  }
};

prototype._didHistoryHeader = function(str ,isHistoryHeader) {
  if (isHistoryHeader) {
    $('.history_header').show().html(str);
    $('.history_list').empty().hide();
  } else {
    $('.history_header').empty().hide();
    $('.history_list').show().html(str);
  }
};

prototype._didCreateHistoryHeader = function() {
  $('#task-add-h1').html(
      [
       '<span class="history_header"></span>',
       '<span class="history_list"></span>'
      ].join('')
   );
   $('#auto-download-tool-controller-server').html(
     [
       '<table id="auto-download-tool-controller-server-list" class="server-contents-list"></table>'
     ].join('')
   );
};

prototype._didPushHistory = function(title) {
  var history = $('.history_list').text();

  if (history) history += " > ";
  history += title;
  this._didHistoryHeader(history, false);
};

prototype._didPopHistory = function() {
  var history = $(".history_list").text();
  var histryArray = history.split(" > ");
  var new_history = "";
  var histryArray_length = histryArray.length - 1;

  for (var i = 0; i < histryArray_length; i += 1) {
    if (i > 0) new_history += " > ";
    new_history += histryArray[i];
  }
  this._didHistoryHeader(new_history, false);
  return histryArray[histryArray.length - 1];
};

prototype._didObjectId = function() {
  var currentObjectIds = ["none"];

  return {
    setCurrentObjectId: function(id) {
      currentObjectIds.push(id);
    },
    getCurrentObjectId: function() {
      return currentObjectIds[currentObjectIds.length - 1];
    },
    getParentObjectId: function() {
      return currentObjectIds[currentObjectIds.length - 2];
    },
    popObjectIds: function() {
      return currentObjectIds.pop();
    }
  }
};

prototype._didAddDstContainerRow = function(container_info) {
  var objectId = container_info.objectId;
  var title = container_info.title;
  var icon;
  var button;

  if (container_info['class'].indexOf("object.container") !== -1) {
    icon = ['<input type="checkbox" class=',objectId,' name="chkbox_adt">'].join('');
    button = ['<input type="button" class="dst_container_class ',objectId,'" value=',title,'>'].join('');
  } else if (container_info['class'].indexOf("object.item.videoItem") !== -1) {
    icon = '<img src="./img/ico_video.png" width="35" height="25" name="item_icon">';
    button = ['<span id=',objectId,' class="item_title_button">',title,'</span>'].join('');
  } else if (container_info['class'].indexOf("object.item.audioItem") !== -1) {
    icon = '<img src="./img/ico_music.png" width="35" height="25" name="item_icon">';
    button = ['<span id=',objectId,' class="item_title_button">',title,'</span>'].join('');
  } else if (container_info['class'].indexOf("object.item.imageItem") !== -1) {
    icon = '<img src="./img/ico_photo.png" width="35" height="25" name="item_icon">';
    button = ['<span id=',objectId,' class="item_title_button">',title,'</span>'].join('');
  }

  $('#auto-download-tool-controller-server-list').append(
      [
       '<tr><td>',
       icon,
       button,
       '</td></tr>'
     ].join('')
   );

  $('.dst_container_class' + '.' + objectId).one('click', dixim.proxy(function() {
    this._didSelectOneCheckbox();
    this._didPushHistory(title);
    this.checkObjectId.setCurrentObjectId(objectId);
    this._didCreateDstContainerTable(objectId);
  }, this));
};

prototype._didAddParentDstContainerRow = function() {
  $('#auto-download-tool-controller-server-list').html(
     [
      '<tr><td>',
      '<img src="./img/ico_dir.png" width="35" height="25" class="dir_icon parent_container" name="parent_dst_icon">',
      '<input type="button" id="parent_dst_container_btn" class="parent_container" value=',_("ctt_ad_wizard_return_parent_container"),'>',
      '</td></tr>'
    ].join('')
  );
};

prototype._didCreateDstRootContainerTableBrowse = function(e){
  var is_complete_getting_data = false;
  var containers = e.args.data.results;
  var containers_length = containers.length;

  this._remain_browse_storage_count -= 1;
  if (this._remain_browse_storage_count == 0) {
    is_complete_getting_data = true;
  }

  for (var i = 0; i < containers_length; i += 1) {
    containers[i].objectId = containers[i].id;
    containers[i].class = (containers[i].class === "container") ? "object.container" : "";
    this._rootContainerNames.push(containers[i].title);
    this._didAddDstContainerRow(containers[i]);
  }
  if (is_complete_getting_data) {
    this._didHistoryHeader(_("ctt_ad_wizard_dst_server_container_list_str"),true);
    $('[type="checkbox"]').on('click', dixim.proxy(function() {
      this._didSelectOneCheckbox();
    }, this));
    this._didSelectOneCheckbox();
  }
};

prototype._didCreateDstRootContainerTableBrowseStorages = function(e){
  var storages = e.args.data.storages;
  var storages_len = storages.length;
  var query = {};

  this._remain_browse_storage_count = storages_len;
  $('#auto-download-tool-controller-server-list').empty();
  this._rootContainerNames = [];
  for (var i = 0; i < storages_len; i += 1) {
    query = {
      id: storages[i].id,
      startingIndex: 0,
      requestedCount: 0,
      filter: "container",
    };
    this._model_transfer.browse(query);
  }
};

prototype._didCreateDstRootContainerTable = function(){
  this._model_transfer.storageList();
};

prototype._didCreateDstContainerTable = function(objectId){
  var browse_query = {
    objectId: objectId,
    browseFlag: "BrowseDirectChildren"
  }

  $('#auto-download-tool-controller-server-list').empty();
  this._didAddParentDstContainerRow();

  browse_query.success = dixim.proxy(function(e) {
    var containers_info = e.args.objects;
    var containers_info_length = containers_info.length;

    for (var i = 0; i < containers_info_length; i += 1) {
      this._didAddDstContainerRow(containers_info[i]);
    }
    $('[type="checkbox"]').on('click', dixim.proxy(function() {
      this._didSelectOneCheckbox();
    }, this));
  }, this);
  browse_query.error = dixim.proxy(function() {
    this._showErrorDialog("ctt_error_dialog_default");
  }, this);
  browse_query.complete = dixim.proxy(function() {
    this._loadingIcon.detach();
    $('[type="button"]').prop('disabled', false);
    $('.parent_container').bind();
    this._didSelectOneCheckbox();
  }, this);
  $('.ui-dialog-buttonset').append(this._loadingIcon);
  $('[type="button"]').prop('disabled', true);
  $('.parent_container').unbind();

  this._model.browse(browse_query);

  $('.parent_container').one('click', dixim.proxy(function() {
    var root_container_name = this._didPopHistory();
    var root_flag = 0;

    this.checkObjectId.popObjectIds();
    for (var j = 0; j < this._rootContainerNames.length; j += 1) {
      if (root_container_name === this._rootContainerNames[j]) {
        this._didCreateDstRootContainerTable();
        root_flag = 1;
      }
    }
    if (! root_flag) {
      this._didCreateDstContainerTable(this.checkObjectId.getCurrentObjectId());
    }
  }, this));
};

prototype._didRenderDialog = function(e) {
  this._didCreateHistoryHeader();
  this._didCreateDstRootContainerTable();
  superConstructor.prototype._didRenderDialog.call(this, e);
};

prototype._didTitlebarCloseButtonClick = function(e) {
  e.preventDefault();
  this._didCancel();
};

prototype._didClickCancel = function() {
  this._didCancel();
};

prototype._didCancel = function() {
  this.dispatchEvent(new Event(CANCEL));
};

prototype._didClickBack = function() {
  this.dispatchEvent(new Event(BACK));
};

prototype._didClickNext = function() {
  this.result.dstContainerId = $('#auto-download-tool-controller-server-list input:checked').prop('class');
  this.dispatchEvent(new Event(NEXT));
};

prototype.getSelectInfo = function() {
  return this.result;
};

/**
 * @method _showErrorDialog
 * @private
 * @param {String} error error ID.
 */
prototype._showErrorDialog = function(error) {
  var dialog;
  var query = {type: error};

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  if (typeof error.title !== "undefined") {
    query.title = error.title;
  }
  if (typeof error.description !== "undefined") {
    query.description = error.description;
  }

  dialog = new ErrorDialog(query);
  dialog.addEventListener('close', this._didCloseModalDialog, this);

  dialog.show();
  this._modal = dialog;
};

prototype._didCloseModalDialog = function(e) {
  var args = e.args;

  if (this._modal === args.dialog) {
    this._modal.destroy();
    this._modal = null;
  }
};

}());
