/**
 * @module ctt
 * @submodule contents-transfer-tool-view
 */
(function() {
var EventTarget = dixim.event.EventTarget;
var $ = dixim.ctt.jQuery;

var ContentsTransferToolView;
var prototype;

/**
 * Root view of the Contents Transfer Tool.
 *
 * @class ContentsTransferToolView
 * @constructor
 * @param {Object} configs
 * @param {String} configs.containerNode
 */
ContentsTransferToolView = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(ContentsTransferToolView, EventTarget);
dixim.ctt.ContentsTransferToolView = ContentsTransferToolView;
prototype = ContentsTransferToolView.prototype;

ContentsTransferToolView.ELEMENT_ID_SEARCH_BOX = 'search';
ContentsTransferToolView.ELEMENT_ID_CONTAINER_LIST = 'container-list';
ContentsTransferToolView.ELEMENT_ID_CONTENT_LIST = 'content';
ContentsTransferToolView.ELEMENT_ID_FOOTER = 'footer';
ContentsTransferToolView.ELEMENT_ID_FOOTER_MENU = 'footer-menu';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 * @param {String} configs.containerNode
 */
prototype._initialize = function(configs) {
  this._containerNode = configs.containerNode;
};

/**
 * @method _createView
 * @private
 */
prototype._createView = function() {
  var v = {};

  v = $(this._containerNode);
  this._header = $('#header', v);
  this._sidebar = $('#sidebar', v);
  this._footer = $('#footer', v);
  this._page = $('#page', v);
  this._view = v;
};

/**
 * @method show
 */
prototype.show = function() {
  if (!!!this._view) {
    this._createView();
  }
  this._view.show();
};

/**
 * @method hide
 */
prototype.hide = function() {
  this._view.hide();
};

}());
