/**
 * @module dixim
 * @submodule server
 */
dixim.namespace('dixim.server');

/**
 * @module server
 * @submodule public-folder-wizard-dialog-controller
 */

(function() {
var PublicFolderWizardDialogController;
var prototype;
var superConstructor;
var EventTarget = dixim.event.EventTarget;
var Event = dixim.event.Event;
var _ = dixim.ctt.ResourceBundle.get;
var PublicFolderWizardSelectFolderDialogView = dixim.server.PublicFolderWizardSelectFolderDialogView;
var MediaTypeDialogView = dixim.server.MediaTypeDialogView;
var ApiClientServer = dixim.common.ApiClientServer;
var Enum = dixim.ctt.util.Enum;
var Template = dixim.ctt.util.Template;

/**
 * @class PublicFolderWizardDialogController
 * @constructor
 * @param {Object} configs
 */
PublicFolderWizardDialogController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

var NEXT = 'next';
var PREV = 'prev';
var SELECT_DIRECTORY = 'selectDirectory';
var EDIT = 'edit';
var ADD = 'add';
var UPDATE = 'update';

var MEDIA_TYPE_DEF = 7;
var SEPARATE = Template.SEPARATE;

var Mode = Enum.fetch('DIALOG');

dixim.inherit(PublicFolderWizardDialogController, EventTarget);
dixim.server.PublicFolderWizardDialogController = PublicFolderWizardDialogController;
prototype = PublicFolderWizardDialogController.prototype;
superConstructor = PublicFolderWizardDialogController.superConstructor;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    this._model = new ApiClientServer();
    this._model.addEventListener('browseFileSystem', this._successBrowseFileSystem, this);
    this._model.addEventListener('browseFileSystemError', this._errorBrowseFileSystem, this);
    this._model.addEventListener('setSharedFolder', this._successSetSharedFolder, this);
    this._model.addEventListener('setSharedFolderError', this._errorSetSharedFolder, this);

    this._title = _('server_public_folder_add_title');
    this._selectFolderView;
    this._mediaTypeView;
    this._currentPath = [];
    this._rootPathList = [];
};

/**
 * @method start
 */
prototype.start = function() {
    this._viewSelectFolder();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._viewSelectFolder = function() {
    var query = {};

    query.title = this._title;
    this._selectFolderView = new PublicFolderWizardSelectFolderDialogView(query);
    this._selectFolderView.addEventListener(SELECT_DIRECTORY, this._clickDirectory, this);
    this._selectFolderView.addEventListener(NEXT, this._clickNext, this);
    this._selectFolderView.addEventListener(PREV, this._clickPrevFolder, this);
    this._selectFolderView.show();

    this._browseFileSystem(this._currentPath);
};

prototype._isMatchArr = function(arr1, arr2) {
    var len1 = arr1.length;
    var len2 = arr2.length;

    if (len1 !== len2) return false;

    for (var i = 0; i < len1; i += 1) {
        if (arr1[i] !== arr2[i]) return false;
    }
    return true;
};

prototype._isMatchRootPath = function(pathList) {
    var len;
    var rootPathList;

    len = this._rootPathList.length;
    for (var i = 0; i < len; i += 1) {
        rootPathList = $.extend(true, [], this._rootPathList[i]);
        rootPathList.pop();
        if (this._isMatchArr(rootPathList, pathList)) return true;
    }
    return false;
};

prototype._browseFileSystem = function(p) {
    var query = {};
    var pathList = $.extend(true, [], p);
    var len;

    len = pathList.length;
    for (var i = 0; i < len; i += 1) {
        if (pathList[i] === '') {
            pathList.splice(i, 1);
            i -= 1;
        }
    }

    if (this._isMatchRootPath(pathList)) {
        pathList = [];
        this._currentPath = [];
    }

    switch(pathList.length) {
        case 0:
            query.path = '';
            break;
        default:
            if (SEPARATE !== '/') {  // Windows
                query.path = [(pathList[0] === SEPARATE) ? '' : pathList.join(SEPARATE)].join('');
            } else { // Linux
                query.path = [(pathList[0] === SEPARATE) ? '' : SEPARATE, pathList.join(SEPARATE)].join('');
            }
            break;
    }
    query.enableHiddenDir = 0;
    this._model.browseFileSystem(query);
};

prototype._successBrowseFileSystem = function(e) {
    var dirs = e.args.dirs;
    var dirsLen = dirs.length;
    var request = e.args.session.request;

    if (this._rootPathList.length === 0 && request.path === '') {
        this._rootPathList = $.extend(true, [], dirs);
        for (var i = 0; i < this._rootPathList.length; i += 1) {
            this._rootPathList[i] = this._rootPathList[i].split(SEPARATE);
            for (var j = 0; j < this._rootPathList[i].length; j += 1) {
                if (this._rootPathList[i][j] === '') {
                    this._rootPathList[i].splice(j, 1);
                    j -= 1;
                }
            }
        }
    }

    this._selectFolderView.cleanList();

    if (this._isMatchRootPath(this._currentPath) || this._currentPath.length === 0) {
        this._selectFolderView.setReturnButton();
    } else {
        this._selectFolderView.setReturnButton(PREV);
    }

    if (dirsLen === 0) {
         this._selectFolderView.noRow();
         return;
    }

    for (var i = 0; i < dirsLen; i += 1) {
        this._selectFolderView.addRow(dirs[i]);
    }
    this._selectFolderView.changeCheckbox();
};

prototype._errorBrowseFileSystem = function(e) {
    this._selectFolderView.error(_('server_public_folder_add_fetch_error'));
};

prototype._clickDirectory = function(e) {
    var pathList = (e.args.path === SEPARATE) ? [SEPARATE]: e.args.path.split(SEPARATE);
    var len = pathList.length;

    for (var i = 0; i < len; i += 1) {
        if (pathList[i] !== '') {
            this._currentPath.push(pathList[i]);
        }
    }
    this._browseFileSystem(this._currentPath);
};

prototype._clickNext = function(e) {
    var pathList = e.args.path.split(SEPARATE);
    var len = pathList.length;

    for (var i = 0; i < len; i += 1) {
        if (pathList[i] !== '') {
            this._currentPath.push(pathList[i]);
        }
    }
    this._viewSelectMediaType();
};

prototype._clickPrevFolder = function() {
    this._currentPath.pop();
    this._browseFileSystem(this._currentPath);
};

prototype._viewSelectMediaType = function() {
    var query = {};

    query.sharedFolder = {
        path: this._currentPath.join(SEPARATE),
        mediaType: MEDIA_TYPE_DEF,
    }
    query.doubleDialog = true;
    query.mode = Mode.ADD;
    this._mediaTypeView = new MediaTypeDialogView(query);
    this._mediaTypeView.addEventListener(ADD, this._clickAdd, this);
    this._mediaTypeView.addEventListener(PREV, this._clickPrev, this);
    this._mediaTypeView.show();
    this._mediaTypeView.setReturnButton(PREV);
};

prototype._clickPrev = function() {
    this._currentPath.pop();
    this._mediaTypeView.hide();
};

prototype._clickAdd = function(e) {
    var query = {};
    var path = $.extend(true, [], this._currentPath);
    var prefix = path[0];

    if (SEPARATE === '/') {  // Linux
        if (prefix === SEPARATE) {
            path[0] = '';
        } else {
            path.unshift('');
        }
    }
    this._mediaTypeView.startIndicator();
    query.sharedFolder = path.join(SEPARATE);
    query.mediaType = e.args.mediaType;
    this._model.setSharedFolder(query);
};

prototype._successSetSharedFolder = function() {
    this._mediaTypeView.stopIndicator();
    this._close();
    this.dispatchEvent(new Event(UPDATE));
};

prototype._errorSetSharedFolder = function(e) {
    var errorCode = e.args.errorCode;

    this._mediaTypeView.stopIndicator();
    switch(errorCode) {
        case 483:
            this._mediaTypeView.error(_(['server_public_folder_add_set_error_', errorCode].join('')));
            break;
        default:
            this._mediaTypeView.error(_('server_public_folder_add_set_error'));
            break;
    }
};

prototype._close = function() {
    if (typeof this._selectFolderView !== 'undefined') this._selectFolderView.hide();
    if (typeof this._mediaTypeView !== 'undefined') this._mediaTypeView.hide();
};

}());
